;-------------------------------------------------------------------------------
; sys_pmu.asm
;
; (c) Texas Instruments 2012, All rights reserved.
;

    SECTION .text:CODE:NOROOT(2)
    ARM


;-------------------------------------------------------------------------------
; Initialize Pmu
; Note: It will reset all counters

    PUBLIC     _pmuInit_

_pmuInit_

        stmfd sp!, {r0}
        ; set control register
        mrc   p15, #0, r0, c9, c12, #0
        orr   r0,  r0, #(1 << 4) + 6 + 1
        mcr   p15, #0, r0, c9, c12, #0
        ; clear flags
        mov   r0,  #0
        mcr   p15, #0, r0, c9, c12, #3
        ; select counter 0 event
        mcr   p15, #0, r0, c9, c12, #5 ; select counter
        mov   r0,  #0x11
        mcr   p15, #0, r0, c9, c13, #1 ; select event
        ; select counter 1 event
        mov   r0,  #1
        mcr   p15, #0, r0, c9, c12, #5 ; select counter
        mov   r0,  #0x11
        mcr   p15, #0, r0, c9, c13, #1 ; select event
        ; select counter 2 event
        mov   r0,  #2
        mcr   p15, #0, r0, c9, c12, #5 ; select counter
        mov   r0,  #0x11
        mcr   p15, #0, r0, c9, c13, #1 ; select event
        ldmfd sp!, {r0}
        bx    lr

;-------------------------------------------------------------------------------
; Enable Counters Global [Cycle, Event [0..2]]
; Note: It will reset all counters

    PUBLIC     _pmuEnableCountersGlobal_

_pmuEnableCountersGlobal_

        stmfd sp!, {r0}
        mrc   p15, #0, r0, c9, c12, #0
        orr   r0,  r0, #7
        mcr   p15, #0, r0, c9, c12, #0
        ldmfd sp!, {r0}
        bx    lr

;-------------------------------------------------------------------------------
; Disable Counters Global [Cycle, Event [0..2]]

    PUBLIC     _pmuDisableCountersGlobal_

_pmuDisableCountersGlobal_

        stmfd sp!, {r0}
        mrc   p15, #0, r0, c9, c12, #0
        bic   r0,  r0, #1
        mcr   p15, #0, r0, c9, c12, #0
        ldmfd sp!, {r0}
        bx    lr

;-------------------------------------------------------------------------------
; Reset Cycle Counter

    PUBLIC     _pmuResetCycleCounter_

_pmuResetCycleCounter_

        stmfd sp!, {r0}
        mrc   p15, #0, r0, c9, c12, #0
        orr   r0,  r0, #4
        mcr   p15, #0, r0, c9, c12, #0
        ldmfd sp!, {r0}
        bx    lr

;-------------------------------------------------------------------------------
; Reset Event Counters [0..2]

    PUBLIC     _pmuResetEventCounters_

_pmuResetEventCounters_

        stmfd sp!, {r0}
        mrc   p15, #0, r0, c9, c12, #0
        orr   r0,  r0, #2
        mcr   p15, #0, r0, c9, c12, #0
        ldmfd sp!, {r0}
        bx    lr

;-------------------------------------------------------------------------------
; Reset Cycle Counter abd Event Counters [0..2]

    PUBLIC     _pmuResetCounters_

_pmuResetCounters_

        stmfd sp!, {r0}
        mrc   p15, #0, r0, c9, c12, #0
        orr   r0,  r0, #6
        mcr   p15, #0, r0, c9, c12, #0
        ldmfd sp!, {r0}
        bx    lr

;-------------------------------------------------------------------------------
; Start Counters [Cycle, 0..2]

    PUBLIC     _pmuStartCounters_

_pmuStartCounters_

        mcr   p15, #0, r0, c9, c12, #1
        bx    lr

;-------------------------------------------------------------------------------
; Stop Counters [Cycle, 0..2]

    PUBLIC     _pmuStopCounters_

_pmuStopCounters_

        mcr   p15, #0, r0, c9, c12, #2
        bx    lr

;-------------------------------------------------------------------------------
; Set Count event

    PUBLIC     _pmuSetCountEvent_

_pmuSetCountEvent_

        lsr   r0,  r0, #1
        mcr   p15, #0, r0, c9, c12, #5 ; select counter
        mcr   p15, #0, r1, c9, c13, #1 ; select event
        bx    lr

;-------------------------------------------------------------------------------
; Get Cycle Count

    PUBLIC     _pmuGetCycleCount_

_pmuGetCycleCount_

        mrc   p15, #0, r0, c9, c13, #0
        bx    lr

;-------------------------------------------------------------------------------
; Get Event Counter Count Value

    PUBLIC     _pmuGetEventCount_

_pmuGetEventCount_

        lsr   r0,  r0, #1
        mcr   p15, #0, r0, c9, c12, #5 ; select counter
        mrc   p15, #0, r0, c9, c13, #2 ; read event counter
        bx    lr

;-------------------------------------------------------------------------------
; Get Overflow Flags

    PUBLIC     _pmuGetOverflow_

_pmuGetOverflow_

        mrc   p15, #0, r0, c9, c12, #3 ; read overflow
        mov   r1,  #0
        mcr   p15, #0, r1, c9, c12, #3 ; clear flags
        bx    lr

;-------------------------------------------------------------------------------

        END
